@@ -1,4 +1,7 @@
-class GraphNode:
+from ..node import Node
+
+
+class GraphNode(Node):
     def __init__(self, val=0, neighbors=None):
-        self.val = val
+        super().__init__(val)
         self.neighbors = neighbors if neighbors is not None else []
