@@ -1,5 +1,8 @@
-class BinaryTreeNode:
+from ..node import Node
+
+
+class BinaryTreeNode(Node):
     def __init__(self, val=0, left=None, right=None):
-        self.val = val
+        super().__init__(val)
         self.left = left
         self.right = right
